(**************** MODULO DE IMPLEMENTACION DE BIBLIO ***********)

IMPLEMENTATION MODULE Biblio;
IMPORT cadenas;(*MODULO DE MANEJO DE CADENAS*)
IMPORT IO;(*LIBRERIA DE E/S*)
IMPORT FIO;(*LIBRERIA DE MANEJO DE FICHEROS*)
IMPORT Window;(*LIBRERIA DE MANEJO DE PANTALLA*)
FROM Storage IMPORT ALLOCATE,DEALLOCATE,Available;(* EXPORTACION DE PROCEDIMIENTOS PARA LAS VARIABLES DINAMICAS*)

(* DECLARACION DE TIPO OPACO *)

TYPE
   PuntLibro=POINTER TO NodoLibro;
   PuntPrestamo=POINTER TO NodoPrestamo;
   NodoLibro=RECORD
             Titulo:cadenas.Tcadenas;
             Autor:cadenas.Tcadenas;
             Ejemplares:CARDINAL;
             Sig:PuntLibro;
             Prestamo:PuntPrestamo;
             END;
   NodoPrestamo=RECORD
                Dni:cadenas.TDni;
                Nombre:cadenas.Tcadenas;
                Sig:PuntPrestamo;
                END;

(* FUNCION PARA LA CREACION DE LA ESTRUCTURA *)

PROCEDURE Crear(VAR Biblio:PuntLibro);
BEGIN
      Biblio:=NIL;
END Crear;

(* FUNCION PARA EL CHEQUEO DE ESTADO DE LA ESTRUCTURA *)
(* DEVUELVE:                                          *)
(*           TRUE---- SI LA ESTRUCTURA ESTA VACIA     *)
(*           FALSE--- SI LA ESTRUCTURA NO ESTA VACIA  *)



PROCEDURE Vacio(Biblio:PuntLibro):BOOLEAN;
BEGIN
    RETURN (Biblio=NIL)
END Vacio;

(* FUNCION QUE CHEQUEA SI EXISTE EL LIBRO PASADO COMO PARAMETRO *)
(* DEVUELVE:                                                    *)
(*               TRUE---- EL LIBRO ESTA EN LA ESTRUCTURA        *)
(*               FALSE -- EL LIBRO NO ESTA EN LA ESTRUCTURA     *)


PROCEDURE EstaLibro(Biblio:PuntLibro;titulo:cadenas.Tcadenas):BOOLEAN;
VAR
  Encontrado:BOOLEAN;
  aux:PuntLibro;

BEGIN
    Encontrado:=FALSE;
    IF Vacio(Biblio) THEN
       RETURN(Encontrado);
    ELSE
       aux:=Biblio;
       WHILE (aux<>NIL) AND NOT(Encontrado) DO
             IF cadenas.Comparar(aux^.Titulo,titulo)=0 THEN
                Encontrado:=TRUE;
             ELSE
                aux:=aux^.Sig;
             END;
       END;
       RETURN(Encontrado);
    END;
END EstaLibro;

(* FUNCION QUE CHEQUEA QUE EL DNI DADO POR PARAMETRO EXISTE EN LA ESTRUCTURA *)
(* DEVUELVE:                                                                 *)
(*           TRUE ----- EL DNI EXISTE EN LA ESTRUCTURA                       *)
(*           FALSE ---- EL DNI NO ESTA EN LA ESTRUCTURA                      *)

PROCEDURE EstaDni(prestamo:PuntPrestamo;dni:cadenas.TDni):BOOLEAN;
VAR
  Encontrado:BOOLEAN;
  aux2:PuntPrestamo;
BEGIN
    Encontrado:=FALSE;
    aux2:=prestamo;
    WHILE (aux2<>NIL) AND NOT(Encontrado) DO
          IF cadenas.CompaDni(aux2^.Dni,dni) THEN
             Encontrado:=TRUE;
          ELSE
             aux2:=aux2^.Sig;
          END;
    END;
    RETURN(Encontrado);
END EstaDni;

(* PROCEDIMIENTO DE INSERCION DE NUEVO DATO EN LA LISTA *)

PROCEDURE Insertar(VAR Biblio:PuntLibro;titulo,autor:cadenas.Tcadenas;existencia:CARDINAL;VAR er:TError);
VAR
  aux,pre:PuntLibro;
  nuevo:PuntLibro;
  Encontrado:BOOLEAN;
BEGIN
    aux:=NIL;
    IF NOT(Available(SIZE(nuevo^))) THEN(*CHEQUEO DE MEMORIA*)
       er:=Nomemoria;
    ELSE
       (*COMPROBACION DE LA EXISTENCIA DEL LIBRO EN LA LISTA *)
       IF EstaLibro(Biblio,titulo) THEN

          (* SI EXISTE LO BUSCA Y LE SUMA LOS NUEVOS EJEMPLARES PASADOS COMO PARAMETRO *)

          aux:=Biblio;
          WHILE (aux<>NIL) AND  NOT(Encontrado) DO
                IF (cadenas.Comparar(aux^.Titulo,titulo)=0) THEN
                   Encontrado:=TRUE;
                ELSE
                   aux:=aux^.Sig;
                END;
           END;
           aux^.Ejemplares:=aux^.Ejemplares+existencia;
       ELSE
          (* SI NO EXISTE CREA EL NUEVO NODO *)
          NEW(nuevo);
          nuevo^.Titulo:=titulo;
          nuevo^.Autor:=autor;
          nuevo^.Ejemplares:=existencia;
          nuevo^.Prestamo:=NIL;
          nuevo^.Sig:=NIL;
          (* LE BUSCA LA POSICION PARA INSERTARLO POR ORDEN ALFABETICO *)

          IF Vacio(Biblio) THEN
             Biblio:=nuevo;
          ELSE
             pre:=NIL;
             aux:=Biblio;
             WHILE (aux<>NIL) AND (cadenas.Comparar(aux^.Titulo,titulo)=2) DO
                pre:=aux;
                aux:=aux^.Sig;
             END;
             IF pre=NIL THEN
                nuevo^.Sig:=aux;
                Biblio:=nuevo;
             ELSE
                pre^.Sig:=nuevo;
                nuevo^.Sig:=aux;
             END;
           END;
       END;
    END;

END Insertar;

(*PROCEDIMIENTO DE PRESTAMO DE LIBRO *)

PROCEDURE Presta(VAR Biblio:PuntLibro;titulo,nombre:cadenas.Tcadenas;dni:cadenas.TDni;VAR er:TError);
VAR
aux:PuntLibro;
aux2:PuntPrestamo;
nuevo,pre:PuntPrestamo;
ok:BOOLEAN;

BEGIN
    aux:=NIL;
    IF NOT(Available(SIZE(nuevo^))) THEN
       er:=Nomemoria;
    ELSE
       IF Vacio(Biblio) THEN
          er:=ListaVacia;
       ELSE
          (*CHEQUEA LA EXISTENCIA DEL LIBRO A PRESTAR*)

          IF NOT(EstaLibro(Biblio,titulo)) THEN
             er:=Libronoesta;
           ELSE
             (*BUSQUEDA DEL LIBRO A PRESTAR*)
                aux:=Biblio;
                ok:=FALSE;
                WHILE (aux<>NIL) AND NOT(ok) DO
                      IF cadenas.Comparar(aux^.Titulo,titulo)=0 THEN
                         ok:=TRUE;
                      ELSE
                         aux:=aux^.Sig;
                      END;
                END;

                (*CHEQUEO DE EJEMPLARES DISPONIBLES PARA PRESTAR *)

                IF aux^.Ejemplares=0 THEN
                   er:=NoejemplaresDisponibles;
                ELSE
                (* CREA EL NODO DEL CLIENTE AL QUE SE LE PRESTA *)

                NEW(nuevo);
                nuevo^.Dni:=dni;
                nuevo^.Nombre:=nombre;
                nuevo^.Sig:=NIL;
                aux^.Ejemplares:=aux^.Ejemplares-1;

                (* BUSCA SU POSICION PARA INSERTARLO *)

                IF aux^.Prestamo=NIL THEN
                   IO.WrStr ("aux2:=NIL");
                   aux^.Prestamo:=nuevo;
                ELSE
                   aux2:=aux^.Prestamo;
                   pre:=NIL;
                   WHILE (aux2<>NIL) AND (cadenas.Comparar(aux2^.Nombre,nombre)=2) DO
                         pre:=aux2;
                         aux2:=aux2^.Sig;
                   END;
                   IF pre=NIL THEN
                      aux2^.Sig:=nuevo;
                   ELSE
                      pre^.Sig:=nuevo;
                      nuevo^.Sig:=aux2;
                   END;
                END;
             END;
          END;
       END;
    END;

END Presta;

(* PROCEDIMIENTO PARA DELVOLVER UN EJEMPLAR *)

PROCEDURE Devolver(VAR Biblio:PuntLibro;dni:cadenas.TDni;titulo:cadenas.Tcadenas;VAR er:TError);
VAR
  aux:PuntLibro;
  aux2:PuntPrestamo;
  pre:PuntPrestamo;
  Encontrado:BOOLEAN;


BEGIN
    Encontrado:=FALSE;
    IF Vacio(Biblio) THEN
       er:=ListaVacia;
    ELSE
       (* CHEQUEO DE LA EXISTENCIA DEL LIBRO A DEVOLVER *)

       IF NOT (EstaLibro(Biblio,titulo)) THEN
          er:=Libronoesta;
       ELSE
          aux:=Biblio;
          Encontrado:=FALSE;
          (* BUSQUEDA DEL LIBRO A DEVOLVER *)

          WHILE (aux<>NIL) AND NOT(Encontrado) DO
                IF cadenas.Comparar(aux^.Titulo,titulo)=0 THEN
                   Encontrado:=TRUE;
                ELSE
                   aux:=aux^.Sig;
                END;
          END;

          (*CHEQUEO DE EL DNI DADO COMO PARAMETRO*)

          IF NOT(EstaDni(aux^.Prestamo,dni)) THEN
             er:=Dninoesta;
          ELSE
             (* EN CASO AFIRMATIVO AUMENTA LOS EJEMPLARES DISPONEBLES *)
             aux^.Ejemplares:=aux^.Ejemplares+1;
             aux2:=aux^.Prestamo;
             pre:=NIL;
             Encontrado:=FALSE;

             (* BUSCA EL NODO CORRECTO Y LO ELIMINA DE LA LISTA *)

             WHILE (aux2<>NIL) AND NOT(Encontrado) DO
                   IF cadenas.CompaDni(aux2^.Dni,dni) THEN
                      Encontrado:=TRUE;
                   ELSE
                      pre:=aux2;
                      aux2:=aux2^.Sig;
                   END;
            END;
            IF pre=NIL  THEN
                IF aux2^.Sig<> NIL THEN
                  aux^.Prestamo:=aux2^.Sig;
                  DISPOSE(aux2);
                ELSE
                  aux^.Prestamo:=NIL;
                  DISPOSE(aux2);
                END;
            ELSE
                pre^.Sig:=aux2^.Sig;
                DISPOSE(aux2);
             END;
          END;
        END;
    END;
END Devolver;

(* PROCEDIMIENTO PARA REGALO *)

PROCEDURE Regalo(VAR Biblio:PuntLibro;titulo:cadenas.Tcadenas;VAR er:TError);
VAR
  aux,prelibro:PuntLibro;
  aux2,pre:PuntPrestamo;
  Encontrado:BOOLEAN;
BEGIN
    IF Vacio(Biblio) THEN
       er:=ListaVacia;
    ELSE
       (*CHEQUEO DE LA EXISTENCIA DEL LIBRO A REGALAR *)

       IF NOT(EstaLibro(Biblio,titulo)) THEN
          er:=Libronoesta;
       ELSE
          (* BUSCA EL LIBRO A REGALAR *)
          Encontrado:=FALSE;
          aux:=Biblio;
          prelibro:=NIL;
          WHILE (aux<>NIL) AND NOT(Encontrado) DO
                IF (cadenas.Comparar(aux^.Titulo,titulo)=0) THEN
                   Encontrado:=TRUE;
                ELSE
                   prelibro:=aux;
                   aux:=aux^.Sig;
                END;
          END;
          (* COMPRUEBA DE QUE HAY EJEMPLARES PRESTADOS Y LOS ELIMINA DE LA LISTA*)

          IF aux^.Prestamo<>NIL THEN
             aux2:=aux^.Prestamo;
             WHILE aux2<>NIL DO
                   pre:=NIL;
                   IF aux2^.Sig<>NIL THEN
                      aux^.Prestamo:=aux2^.Sig;
                      DISPOSE(aux2);
                   ELSE
                      aux^.Prestamo:=NIL;
                      DISPOSE(aux2);
                   END;
                   aux2:=aux^.Prestamo;
              END;
          END;

          (*CHEQUEA DE QUE RESTAN MAS EJEMPLARES,EN EL CASO CONTRARIO ELIMINA EL*)
          (*LIBRO DE LA LISTA*)

          IF aux^.Ejemplares=0 THEN
             IF prelibro=NIL THEN
                IF aux^.Sig=NIL THEN
                   Biblio:=NIL;
                   DISPOSE(aux);
                ELSE
                   Biblio:=aux^.Sig;
                   DISPOSE(aux);
                END;
             ELSE
                prelibro^.Sig:=aux^.Sig;
                DISPOSE(aux);
             END;
           END;
       END;
     END;

END Regalo;
(* PROCEDIMIENTO PARA EL LISTADO DEL TAD*)

PROCEDURE Listar(Biblio:PuntLibro;VAR er:TError);

(*PROCEDIMIENTO LOCAL DE GENERACION DE PANTALLA*)

PROCEDURE GeneraPantalla();
VAR
  i,x:CARDINAL;
BEGIN
    Window.Clear();
    x:=1;
    Window.GotoXY(1,x);
    IO.WrStr("");
    FOR i:=2 TO 78 DO
        Window.GotoXY(i,x);
        IO.WrStr("");
    END;
    Window.GotoXY(79,1);
    IO.WrStr("");
    Window.GotoXY(1,2);IO.WrStr("");
    Window.GotoXY(27,2);IO.WrStr("LISTADO DE BIBLIOTECA");
    Window.GotoXY(79,2);IO.WrStr("");
    Window.GotoXY(1,3);IO.WrStr("");
    x:=3;
    FOR i:=2 TO 78 DO
        Window.GotoXY(i,x);
        IO.WrStr("");
    END;
    Window.GotoXY(i,x);IO.WrStr("");
    FOR x:=4 TO 24 DO
        Window.GotoXY(1,x);IO.WrStr("");
        Window.GotoXY(79,x);IO.WrStr("");
    END;
    Window.GotoXY(1,25);IO.WrStr("");
    Window.GotoXY(1,23);IO.WrStr("");
    Window.GotoXY(79,25);IO.WrStr("");
    Window.GotoXY(79,23);IO.WrStr("");

    FOR i:=2 TO 78 DO
        Window.GotoXY(i,25);
        IO.WrStr("");
        Window.GotoXY(i,23);
        IO.WrStr("");

    END;
END GeneraPantalla;

(* PROCEDIMIENTO LOCAL QUE COMPRUEBA EL ESTADO DE LA PANTALLA Y DETERMINA*)
(* SI DEBE BORRARSE O NO *)

PROCEDURE Esfin(VAR  x:CARDINAL);
VAR
  pausa:CHAR;
BEGIN
    IF x=23 THEN

       (*PREGUNTA Y ESPERA,PARA BORRAR LA PANTALLA *)

       Window.GotoXY(2,24);IO.WrStr("Mas...");
       IO.RdLn;
       pausa:=IO.RdKey();IO.RdLn;
       x:=4;
       GeneraPantalla();
    END;
END Esfin;


(***** CUERPO PRINCIPAL DEL PROCEDIMIENTO *******)

VAR
  aux:PuntLibro;
  aux2:PuntPrestamo;
  x,i:CARDINAL;
BEGIN
    aux:=Biblio;
    x:=4;
    (* COMPRUEBA QUE LA ESTRUCTURA NO ESTA VACIA*)
    IF Vacio(Biblio) THEN
       er:=ListaVacia;
    ELSE
       (* GENERA LA PANTALLA PRINCIPAL*)
       GeneraPantalla();
        WHILE (aux<>NIL) DO
            (* MIENTRAS HALLA ELEMENTOS LOS IMPRIME EN PANTALLA*)
            Esfin(x);
            Window.GotoXY(79,x);IO.WrStr("");
            Window.GotoXY(1,x);IO.WrStr("");
            FOR i:=2 TO 78 DO
                Window.GotoXY(i,x);
                IO.WrStr("");
            END;
            Window.GotoXY(25,x);IO.WrStr(" Libro ");
            INC(x);Esfin(x);
            Window.GotoXY(2,x);IO.WrStr("Titulo:");IO.WrStr(aux^.Titulo);
            Window.GotoXY(25,x);IO.WrStr("Autor:");IO.WrStr(aux^.Autor);INC(x);Esfin(x);
            Window.GotoXY(15,x);IO.WrStr ("Existencias");IO.WrCard(aux^.Ejemplares,2);INC(x);Esfin(x);

            (* COMPRUEBA LA EXISTENCIA DE PRESTAMOS Y LOS IMPRIME CUANDO CORRESPONDA *)

            IF aux^.Prestamo<>NIL THEN
               Window.GotoXY(1,x);IO.WrStr("");
               Window.GotoXY(79,x);IO.WrStr("");
               FOR i:=2 TO 78 DO
                   Window.GotoXY(i,x);
                   IO.WrStr("");
               END;
               Window.GotoXY(25,x);IO.WrStr(" PRESTADOS A ");INC(x);Esfin(x);
               aux2:=aux^.Prestamo;
               WHILE aux2<>NIL DO
                     Window.GotoXY(2,x);IO.WrStr ("Nombre ");IO.WrStr (aux2^.Nombre);INC(x);Esfin(x);
                     Window.GotoXY(25,x);IO.WrStr("Dni: ");cadenas.EscribirDni(aux2^.Dni);INC(x);Esfin(x);
                     aux2:=aux2^.Sig;
               END;
             END;
             INC(x);
             aux:=aux^.Sig;
       END;
    END;
END Listar;

(**** PROCEDIMIENTO DE DESTRUCCION Y LIBERACION DE MEMORIA *****)

PROCEDURE Destruir(VAR Biblio:PuntLibro);
VAR
  aux:PuntLibro;
  aux2:PuntPrestamo;
BEGIN
    (* BUSCA Y ELIMINA TODOS LOS NODOS DE LA ESTRUCTURA *)

    WHILE Biblio<>NIL  DO
          aux:=Biblio;
          WHILE aux^.Prestamo<>NIL DO
             aux2:=aux^.Prestamo;
             aux^.Prestamo:=aux2^.Sig;
             DISPOSE(aux2);
          END;
          Biblio:=aux^.Sig;
          DISPOSE(aux);
     END;
END Destruir;

(* PROCEDIMIENTO PARA VOLCAR A FICHERO EL TAD *)

PROCEDURE VolcarAfichero(Biblio:PuntLibro;nombre:cadenas.Tcadenas;VAR  er:TError);

(* PROCEDIMIENTO LOCAL QUE CALCULA EL NUMERO DE EJEMPLARES PRESTADOS DEL *)
(* LIBRO PASADO COMO PARAMETRO *)

PROCEDURE EjemplaresPrestados(temp:PuntPrestamo):CARDINAL;
VAR
  i:CARDINAL;
BEGIN
    i:=0;
    WHILE temp<>NIL DO
          INC(i);
          temp:=temp^.Sig;
    END;
    RETURN(i);

END EjemplaresPrestados;

(* CUERPO DEL PROCEDIMIENTO *)

VAR
  fichero:FIO.File;(*MANEJADOR DEL FICHERO*)
  aux:PuntLibro;
  aux2:PuntPrestamo;
  n_prestado:CARDINAL;


BEGIN
    IF Vacio(Biblio) THEN
       er:=ListaVacia;
    ELSE
       aux:=Biblio;
       fichero:=FIO.Create(nombre);(*CREA EL FICHERO Y LO ABRE*)
       WHILE aux<>NIL DO
              (* ESCRIBE LOS DATOS DEL LIBRO *)
              FIO.WrStr(fichero,"Libro:");FIO.WrLn(fichero);
              FIO.WrStr(fichero,aux^.Titulo);FIO.WrLn(fichero);
              FIO.WrStr(fichero,aux^.Autor);FIO.WrLn(fichero);
              FIO.WrCard(fichero,aux^.Ejemplares,1);FIO.WrLn(fichero);

              (*ESCRIBE LOS DATOS DE PRESTAMO SI EXISTEN *)

              IF aux^.Prestamo=NIL THEN
                        FIO.WrStr(fichero,"Prestados:");FIO.WrLn(fichero);
                        FIO.WrCard(fichero,0,1);FIO.WrLn(fichero);
              ELSE
                        FIO.WrStr(fichero,"Prestados:");FIO.WrLn(fichero);
                        n_prestado:=EjemplaresPrestados(aux^.Prestamo);
                        FIO.WrCard(fichero,n_prestado,1);FIO.WrLn(fichero);
                        aux2:=aux^.Prestamo;
                        FIO.WrStr(fichero,"Prestados a:");FIO.WrLn(fichero);
                        WHILE aux2<>NIL DO
                              FIO.WrStr(fichero,aux2^.Nombre);FIO.WrLn(fichero);
                              FIO.WrStr(fichero,aux2^.Dni);FIO.WrLn(fichero);
                              aux2:=aux2^.Sig;
                         END;
              END;
              aux:=aux^.Sig;
        END;
        FIO.Close(fichero);(* CIERRA EL FICHERO *)
     END;

END VolcarAfichero;

(* PROCEDIMIENTO DE CREACION DE TAD A PARTIR DE UN FICHERO *)

PROCEDURE CrearDeFichero(VAR Biblio:PuntLibro;fich:cadenas.Tcadenas; VAR er:TError);
VAR
  fichero:FIO.File;(*MANEJADOR DE FICHERO*)
  titulo,autor,nombre,temp:cadenas.Tcadenas;
  Dni:cadenas.TDni;
  ejemp,nprestado,i:CARDINAL;


BEGIN
    Destruir(Biblio); (*DESTRUYE LA ESTRUCTURA*)
    Crear(Biblio);(* CREA UNA NUEVA ESTRUCTURA LIMPIA*)
    fichero:=FIO.Open(fich);(*ABRE EL FICHERO*)
    FIO.RdStr(fichero,temp);
    WHILE NOT FIO.EOF DO
          (*LEE LOS DATOS*)

          FIO.RdStr(fichero,titulo);
          FIO.RdStr(fichero,autor);
          ejemp:=FIO.RdCard(fichero);FIO.RdStr(fichero,temp);
          nprestado:=FIO.RdCard(fichero);
          (* PROCESA LOS DATOS LEIDOS *)
          ejemp:=ejemp+nprestado;(*SUMA LOS DATOS DE EJEMPLARES Y LOS PRESTADOS PARA QUE DESPUES DE *)
                                (*PROCESAR LOS PRESTADOS EL DATO DE EJEMPLARES TOTALES NO SE VEA AFECTADO*)
          Insertar(Biblio,titulo,autor,ejemp,er);
          IF nprestado>0 THEN
             (* SI HAY EJEMPLARES PRESTADOS LOS LEE*)
              FIO.RdStr(fichero,temp);
             (* LEE LOS DATOS DE PRESTAMO Y LO PROCESA *)

             FOR i:=1 TO nprestado DO
                 FIO.RdStr(fichero,nombre);
                 FIO.RdStr(fichero,Dni);
                 Presta(Biblio,titulo,nombre,Dni,er);
             END;
          END;
          FIO.RdStr(fichero,temp);

     END;
     FIO.Close(fichero);(*CIERRRA EL FICHERO*)
END CrearDeFichero;



END Biblio.











